﻿using Ato.EN.IntegrationServices.CodeGenerationIITRPRFL;
using DataContracts;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.CompilerServices;
using System.Text;
using System.Threading.Tasks;

namespace Ato.EN.IntegrationServices.CodeGenerationIITRPRFL
{
    public partial class IITRPRFL2024Validator
    {
        #region VR.ATO.IITRPRFL.000001

        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public virtual void VRATOIITRPRFL000001(List<ProcessMessageDocument> response, IITRPRFL2024 report, string serviceBuildYear)
        {
            ProcessMessageDocument processMessage;
            bool assertion;

            /*  VR.ATO.IITRPRFL.000001
            Tax Year must be current lodgment year. Current financial year is also allowed when same as service build year to allow pre-production testing.
    
            Legacy Rule Format:
            (^IITRPRFL2 <> CurrentFinancialYear() - 1) AND (^IITRPRFL2 <> CurrentFinancialYear() OR NOT IsNumeric([ServiceBuildYear:0]) OR ^IITRPRFL2 <> AsNumeric([ServiceBuildYear:0]))
            [ServiceBuildYear:0] = Substring(<Envelope><Header><UserMessage><CollaborationInfo><Service>,32,4)

            Technical Business Rule Format:
            (^IITRPRFL2 <> CurrentFinancialYear() - 1) AND (^IITRPRFL2 <> CurrentFinancialYear() OR NOT IsNumeric([ServiceBuildYear:0]) OR ^IITRPRFL2 <> AsNumeric([ServiceBuildYear:0]))
            [ServiceBuildYear:0] = Substring(<Envelope><Header><UserMessage><CollaborationInfo><Service>,32,4)
    
            Data Elements:
    
            ^IITRPRFL2 = IITRPRFL:Rp:Report.TargetFinancial.Year
            */
            assertion = report.IITRPRFL2 != CurrentFinancialYear() - 1 && (report.IITRPRFL2 != CurrentFinancialYear() || !IsNumeric(serviceBuildYear) || report.IITRPRFL2 != AsNumeric(serviceBuildYear));

            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IITRPRFL.000001",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Year of return is invalid",
                    LongDescription = @"The year of return must be the current lodgment year only. Substituted accounting periods (SAP) are not supported by this service.",
                    Location = "/tns:IITRPRFL/tns:Rp/tns:TargetFinancialY",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITRPRFL.000001" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITRPRFL2", Value = report.IITRPRFL2.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
        }

        #endregion
    }
}